#################################################################
#
# File : analoginst.tcl
#
# Author : Martin Hachenberg
#
# Include fuer ein Analoginstrument
#
# Copyright (C) 1998, 1999 Universitaet zu Koeln, Germany.
# Written by Martin Hachenberg (martinh@ikp.uni-koeln.de).
# 
# This file is part of the tkanalog-package.
# 
# tkanalog is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 2 of the License, or (at your
# option) any later version.
# 
# tkanalog is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
# 
# You should have received a copy of the GNU General Public License
# along with tkanalog -- see the file COPYING.  If not, write to the Free
# Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
# MA 02111-1307, USA.
# 
#################################################################

set AInst(list,all) {}
set AInst(list,sb) {}

#################################################################
# Variable erniedrigen

proc AInst_getvar { w } {
    global AInst

    return AInst($w,variable)
}
#################################################################
# Variable erniedrigen

proc AInst_dv { w var } {
    global AInst

    set AInst($w,dv) [expr $AInst($w,mdv) / $AInst($w,sdv).0]
}

#################################################################
# Variable erhoehen

proc AInst_StopSet { btn } {
    global AInst

    if { $btn == $AInst(do,updown) } {
	set AInst($AInst(do,wdw),add$AInst(do,updown)) -1    
    }

    if { $btn == 0 } {
	set AInst($AInst(do,wdw),add1) -1    
	set AInst($AInst(do,wdw),add3) -1    
    }
}
#################################################################
# Variable erhoehen

proc AInst_RunSet {} {
    global AInst
    
    set w $AInst(do,wdw)

    if { $AInst($w,add$AInst(do,updown)) == 1 } {
	set go 1
	set variable [expr $AInst($w,variable) + $AInst(do,dv)]
#	if { $variable < $AInst($w,min) } {
#	    set variable $AInst($w,min)
#	    AInst_StopSet 0
#	    set go 0
#	}
#	if { $variable > $AInst($w,max) } {
#	    set variable $AInst($w,max)
#	    AInst_StopSet 0
#	    set go 0
#	}
	set AInst($w,variable) $variable
	set AInst($w,var) [format "%.1f" $variable]
	if { $go == 1 } {
	    after 40 { AInst_RunSet }
	}
    }
}

#################################################################
# Variable erhoehen

proc AInst_Set {w direction} {
    global AInst

    set lp [lsearch $AInst(list,sb) $w]

    if { $lp < 0 } {
	return
    }

    set wt [lindex $AInst(list,all) $lp]

    set dv [expr $AInst($wt,mdv) / $AInst($wt,sdv).0]
    set AInst(do,dv) [expr $direction*$dv]

    set AInst($wt,add$AInst(do,updown)) 1
    set AInst(do,wdw) $wt

    AInst_RunSet
}

#################################################################
# Variable erhoehen

proc AInst_Down {w} {
    global AInst

    set AInst(do,updown) 1
    AInst_Set $w -1
}

#################################################################
# Variable erniedrigen

proc AInst_Up {w} {
    global AInst

    set AInst(do,updown) 3
    AInst_Set $w 1
}

#################################################################
# Create top frame

proc MakeAnaInst { w wd min max varwert text name } {
    global AInst

    frame $w -relief groove -borderwidt 5

    if { $wd < 150 } {
	set wd 150
    }
    set ht [expr 4 * $wd / 3]

    #initialsieren
    lappend AInst(list,all) $w
    lappend AInst(list,sb) $w.sb
    set AInst($w,variable) $varwert
    set AInst($w,var) [format "%.1f" $varwert]
    set AInst($w,mdv) [expr ($max - $min)]
    set AInst($w,sdv) 300
    set AInst($w,dv) [expr ($max - $min) / $AInst($w,sdv).0]
    set AInst($w,min) $min
    set AInst($w,max) $max
    set h2 [expr $ht / 2]

    frame $w.atemp
    frame $w.stemp
    frame $w.ltemp

    label $w.leer -text ""

    # analoganzeige aufbauen
    analog $w.a -from $min -to $max -variable AInst($w,variable) \
	    -width $wd -height $h2 -text $text

    label $w.n -text $name -relief groove -borderwidth 2
    label $w.v -textvariable AInst($w,var) -relief groove \
	    -borderwidth 2 -width 8 -justify left

    # scale fuer Einstellung der Drehgeschwindigkeit
    scale $w.sc -from 100 -to 1100 -resolution 200 -orient vertical \
	    -variable AInst($w,sdv) -showvalue 0
    
    set scw [lindex [$w.sc configure -width] 4]
    set scw [expr $scw + 2 * [lindex [$w.sc configure -borderwidth] 4]]
    set wd [expr $wd - $scw]

    # drehknopf aufbauen

    canvas $w.sb -width $wd -height $h2 -cursor hand2

    set htd [expr (3 * $h2 / 4)]
    set sbw [expr ($htd / 3)]
    set sbxm [expr ($wd / 2)]
    set sbym [expr ($h2 / 2)]
    set x1 [expr $sbxm - $sbw]
    set x2 [expr $sbxm + $sbw]
    set y1 [expr $sbym - $sbw]
    set y2 [expr $sbym + $sbw]
    $w.sb create arc $x1 $y1 $x2 $y2 -fill gray30 -outline black -width 2 \
	    -start 45 -extent -180
    $w.sb create arc $x1 $y1 $x2 $y2 -fill gray70 -outline black -width 2 \
	    -start 45 -extent 180
    set sbw [expr ($sbw * 8 / 10)]
    set x1 [expr $sbxm - $sbw]
    set x2 [expr $sbxm + $sbw]
    set y1 [expr $sbym - $sbw]
    set y2 [expr $sbym + $sbw]
    $w.sb create oval $x1 $y1 $x2 $y2 -fill black

    pack $w.n $w.v -in $w.ltemp -side left -fill x -expand 1
    pack $w.leer $w.a $w.ltemp -in $w.atemp -side top -expand 1 -fill x
    
    pack $w.sb $w.sc -in $w.stemp -side left

    pack $w.atemp $w.stemp -in $w

    # bindings and required values
    set AInst(do,updown) 0
    set AInst(do,wdw) $w
    set AInst($w,add0) -1
    set AInst($w,add1) -1
    set AInst($w,add3) -1
 
    bind $w.sb <Button-1> { AInst_Down %W }
    bind $w.sb <ButtonRelease-1> { AInst_StopSet 1 }
    bind $w.sb <Button-3> { AInst_Up %W }
    bind $w.sb <ButtonRelease-3> { AInst_StopSet 3 }
    bind $w.sb <Leave> { AInst_StopSet 0 }
}
 
#################################################################
# File : analoginst.tcl                                         #
#################################################################
