/* ******************************************************* */
/*
 * File: tkanalog.c
 *
 * Author: Martin Hachenberg
 *
 * neues Grafik-Widget fuer Tcl/Tk (Analoganzeige)
 *
 * Copyright (C) 1998, 1999 Universitaet zu Koeln, Germany.
 * Written by Martin Hachenberg (martinh@ikp.uni-koeln.de).
 * 
 * This file is part of the tkanalog-package.
 * 
 * tkanalog is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 * 
 * tkanalog is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with tkanalog -- see the file COPYING.  If not, write to the Free
 * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA.
 */
/* ******************************************************* */

#define _TKANALOG_ 1

#if defined(HAVE_CONFIG_H)
#include "config.h"
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#if TIME_WITH_SYS_TIME
#include <sys/time.h>
#include <time.h>
#else
#if HAVE_SYS_TIME_H
#include <sys/time.h>
#else
#include <time.h>
#endif
#endif

#include <X11/X.h>
#include <X11/Xlib.h> 
#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>

#include "tcl.h"
#include "tk.h"

#ifndef M_PI
#define M_PI 3.14159265358979323846
#endif

#define _TCL_7_4_

#if defined(_TCL_7_4_)
#undef _TCL_7_3_
#else
#if defined (_TCL_7_3_)
#undef _TCL_7_4_
#endif
#endif

/* ******************************************************* */

typedef struct {
  Tk_Window tkwin;    /* Ausgabe Widget */
  Display   *display;
  Tcl_Interp *interp;

  Pixmap pix;         /* Pixmap, Grafik des Widget */
  GC gc;              /* Graphic Kontext des Widgets */
  int outpix;

  int width;          /* Breite des Widgets */
  int height;         /* Hoehe des Widgets */
  unsigned int w;              /* Breite der Scala */
  unsigned int h;              /* Hoehe der Scala */
 
  Pixel fgcolor;      /* akt. Vordergrundfarbe des Widgets */
  Pixel bgcolor;      /* akt. Hintergrundfarbe des Widgets */

  int borderwidth;
  Tk_3DBorder fgborder;
  Tk_3DBorder bgborder;
  int linewidth;

  int relief;

  int scale;           /* Flag ob Scalierung anzeigt wird */

  int updatePending;

  double min, max;     /* minima und maxima der Anzeige */

  int midx, midy;      /* Punkt um den der Zeiger lauft (Pixelkoord.) */
  int zl;              /* minimale untere anzeigehoehe fuer Zeiger */
  double zglen;        /* gesamtlaenge des Zeigers */
  double m, n;         /* geradenanpassung fuer Wert->Winkel */
  int ox1, oy1, ox2, oy2;

  int wxmin, wxmax;    /* x-Koord. der Extrema-Warnlampen */
  int wy;              /* y-Koord. der Extrema-Warnlampen */
  unsigned int wd;     /* Durchmesser der Warnlampe */
  Pixel wcolor;        /* Farbe der Warnlampe */

  char *vname;         /* name der trace-variablen */
  double value;

  char *logfname;      /* Dateiname der Logdatei */
  int dolog;           /* Flag ob 'gelogt' werden soll */
  Tk_TimerToken timer;
  time_t startime;
  FILE *logfp;

  int setamin, setamax;
  double amin, amax;
  char *cmdamin;
  char *cmdamax;
  char *cmdreset;
  int alarmflag;
  int resetflag;

  char *text;          /* Text der in der Anzeige erscheinen soll */
  int tx, ty;
  int dotxt;

} Tk_WAnalog;

static Tk_ConfigSpec configSpecs[] = {
  {TK_CONFIG_BORDER, "-background", "background",
   "Background", "white", Tk_Offset(Tk_WAnalog, bgborder),
   TK_CONFIG_COLOR_ONLY, (Tk_CustomOption *) NULL},
  {TK_CONFIG_BORDER, "-background", "background",
   "Background", "white", Tk_Offset(Tk_WAnalog, bgborder),
   TK_CONFIG_MONO_ONLY, (Tk_CustomOption *) NULL},
  {TK_CONFIG_SYNONYM, "-bg", "background",
   (char *) NULL, (char *) NULL, 0, 0, (Tk_CustomOption *) NULL},

  {TK_CONFIG_BORDER, "-foreground", "foreground",
   "Foreground", "black", Tk_Offset (Tk_WAnalog, fgborder),
   TK_CONFIG_COLOR_ONLY, (Tk_CustomOption *) NULL},
  {TK_CONFIG_BORDER, "-foreground", "foreground",
   "Foreground", "black", Tk_Offset (Tk_WAnalog, fgborder),
   TK_CONFIG_MONO_ONLY, (Tk_CustomOption *) NULL},
  {TK_CONFIG_SYNONYM, "-fg", "foreground",
   (char *) NULL, (char *) NULL, 0, 0, (Tk_CustomOption *) NULL},

  {TK_CONFIG_PIXELS, "-width", "width", "Width", "160", 
   Tk_Offset(Tk_WAnalog, width), 0, (Tk_CustomOption *) NULL},

  {TK_CONFIG_PIXELS, "-height", "height", "Height", "120", 
   Tk_Offset(Tk_WAnalog, height), 0, (Tk_CustomOption *) NULL},

  {TK_CONFIG_PIXELS, "-borderwidth", "borderWidth", "BorderWidth", "3", 
   Tk_Offset(Tk_WAnalog, borderwidth), 0, (Tk_CustomOption *) NULL},
  
  {TK_CONFIG_RELIEF, "-relief", "relief", "Relief", "sunken", 
   Tk_Offset(Tk_WAnalog, relief), 0, (Tk_CustomOption *) NULL},

  {TK_CONFIG_DOUBLE, "-from", "from", "From", "0.0", 
   Tk_Offset(Tk_WAnalog, min), 0, (Tk_CustomOption *) NULL},

  {TK_CONFIG_DOUBLE, "-to", "to", "To", "10.0", 
   Tk_Offset(Tk_WAnalog, max), 0, (Tk_CustomOption *) NULL},
  
  {TK_CONFIG_STRING, "-variable", "variable", "Variable", "", 
   Tk_Offset(Tk_WAnalog, vname), 0, (Tk_CustomOption *) NULL},

  {TK_CONFIG_STRING, "-text", "text", "Text", "", 
   Tk_Offset(Tk_WAnalog, text), 0, (Tk_CustomOption *) NULL},
  
  {TK_CONFIG_END, (char *) NULL, (char *) NULL,
   (char *) NULL, (char *) NULL, 0, 0,(Tk_CustomOption *) NULL}

} ;

/* ******************************************************* */
/* extern variables */
/* none */

/* global variables */
/* none */

/* local variables */
/* none */

/* extern functions */

/* global functions */

int Tk_AnalogWidgetCreate ( ClientData , Tcl_Interp * , int , char * [] );

/* local functions */

static char *AnalogTextVarProc (ClientData clientData,
				Tcl_Interp *interp, char *name1, char *name2,
				int flags);

static int Tk_AnalogConfigure ( Tcl_Interp * , Tk_WAnalog * , 
			       int , char *[] , int );
static void Tk_AnalogDisplay ( ClientData  );
static void Tk_AnalogEventProc ( ClientData , XEvent * );
static void Tk_AnalogWidgetDestroy ( ClientData );

static int mrint ( double );
static void plotScale ( Tk_WAnalog * , int , int );
static void plotZeiger ( Tk_WAnalog * , int , int );

static int plotAnalogScale ( Tk_WAnalog * , Tcl_Interp * );
static double mpow10 ( int  );
static void doscale (  Tk_WAnalog * , double , char * );
static int plotScaleLine ( Tk_WAnalog * , double  , double  , int ,
			   int * , int * , int * , int * );
static int DoAlarmCommand ( Tcl_Interp * , char * );
static void StartLog ( ClientData );
static void DoLog ( ClientData );

/*********************************************************
 * Pixmap auf Bildschirm ausgeben
 */

static int Tk_AnalogWidgetCmd ( ClientData clientData , Tcl_Interp *interp ,
			       int argc , char *argv[] )
{
  Tk_WAnalog *anPtr = (Tk_WAnalog *) clientData;
  int result = TCL_OK;
  int i;
  int asmin = 0;
  int asmax = 0;
  double dtmp;
  char *endptr;
  char *alarmcmd = NULL;

  if (argc < 2) {
    Tcl_AppendResult(interp, "wrong # args: shold be \"",
		     argv[0], " option ?arg arg ...?\"",
		     (char *) NULL);
    return(TCL_ERROR);
  }

  Tk_Preserve((ClientData) anPtr);

  if (strcmp(argv[1], "configure") == 0) {
    if (argc == 1) {
      result = Tk_ConfigureInfo(interp, anPtr->tkwin,
				configSpecs, (char *) anPtr,
				(char *) NULL, 0);
    } else if (argc == 3) {
      result = Tk_ConfigureInfo(interp, anPtr->tkwin,
				configSpecs, (char *) anPtr,
				argv[2], 0);
    } else {
      result = Tk_AnalogConfigure(interp, anPtr, argc-2, argv+2,
				  TK_CONFIG_ARGV_ONLY);
    }

    Tk_DoWhenIdle(Tk_AnalogDisplay, (ClientData) anPtr);
    anPtr->updatePending = 1;
  } else if (strcmp (argv[1], "alarm") == 0) {
    if (argc < 4) {
      Tcl_AppendResult(interp, "wrong # args: shold be \"",
		       argv[0], " alarm ? arg arg ...?\"",
		       (char *) NULL);
      Tk_Release((ClientData) anPtr);
      return(TCL_ERROR);
    }

    for (i = 2; (i < argc); i++) {
      if ( (strcmp(argv[i], "-min") == 0) && (i+1 < argc) ) {
	if (strcmp(argv[i], "off") == 0) {
	  if (anPtr->setamin) {
	    if (anPtr->cmdamin != NULL) {
	      free(anPtr->cmdamin);
	    }
	    anPtr->setamin = 0;
	  }
	} else {
	  asmin = 1;
	  anPtr->amin = strtod(argv[i+1], &endptr);
	}
	i++;
      } else if ( (strcmp(argv[i], "-max") == 0) && (i+1 < argc) ) {
	if (strcmp(argv[i], "off") == 0) {
	  if (anPtr->setamax) {
	    if (anPtr->cmdamax != NULL) {
	      free(anPtr->cmdamax);
	    }
	    anPtr->setamax = 0;
	  }
	} else {
	  asmax = 1;
	  anPtr->amax = strtod(argv[i+1], &endptr);
	}
	i++;
      } else if ( (strcmp(argv[i], "-command") == 0) && (i+1 < argc) ) {
	alarmcmd = argv[i+1];
      }

      if (asmin) {
	if (alarmcmd != NULL) {
	  if (anPtr->cmdamin != NULL) {
	    free(anPtr->cmdamin);
	  }
	  /* String kopieren */
	  anPtr->cmdamin = (char *) malloc( (strlen(alarmcmd)+1) 
					    * sizeof(char) );
	  if (anPtr->cmdamin != NULL)
	    strcpy(anPtr->cmdamin, alarmcmd);
	}
	anPtr->setamin = 1;
      }
      if (asmax) {
	if (alarmcmd != NULL) {
	  if (anPtr->cmdamax != NULL) {
	    free(anPtr->cmdamax);
	  }
	  /* String kopieren */
	  anPtr->cmdamax = (char *) malloc( (strlen(alarmcmd)+1) 
					    * sizeof(char) );
	  if (anPtr->cmdamin != NULL)
	    strcpy(anPtr->cmdamax, alarmcmd);
	}
	anPtr->setamax = 1;
      }
      if ( (anPtr->setamin) && (anPtr->setamax) )
	if (anPtr->amin > anPtr->amax) {
	  alarmcmd = anPtr->cmdamin;
	  anPtr->cmdamin = anPtr->cmdamax;
	  anPtr->cmdamax = alarmcmd;
	  dtmp = anPtr->amin;
	  anPtr->amin = anPtr->amax;
	  anPtr->amax = dtmp;
	}
      anPtr->alarmflag = anPtr->resetflag = 0;
      plotAnalogScale(anPtr, interp);
      Tk_DoWhenIdle(Tk_AnalogDisplay, (ClientData) anPtr);
      anPtr->updatePending = 1;
    }
  } else if (strcmp (argv[1], "reset") == 0) {
    if (argc != 3) {
      Tcl_AppendResult(interp, "wrong # args: shold be \"",
		       argv[0], " reset ?arg?\"",
		       (char *) NULL);
      Tk_Release((ClientData) anPtr);
      return(TCL_ERROR);
    }

    if (strlen(argv[2])) {
      if (anPtr->cmdreset != NULL) {
	free(anPtr->cmdreset);
      }
      /* String kopieren */
      anPtr->cmdreset = (char *) malloc( (strlen(argv[2])+1) 
					* sizeof(char) );
      if (anPtr->cmdreset != NULL)
	strcpy(anPtr->cmdreset, argv[2]);
      anPtr->resetflag = 1;
    }
  } else if (strcmp (argv[1], "log") == 0) {
    if (argc != 3) {
      Tcl_AppendResult(interp, "wrong # args: shold be \"",
		       argv[0], " log ?arg?\"",
		       (char *) NULL);
      Tk_Release((ClientData) anPtr);
      return(TCL_ERROR);
    }

    if ( argv[2][0] == '@' ) {
      anPtr->logfname = (char *) malloc( (strlen(argv[2])+1) * sizeof(char) );
      if (anPtr->logfname != NULL) {
	for (i = 0; (i < strlen(argv[2])); i++)
	  anPtr->logfname[i] = argv[2][i+1];
	anPtr->logfp = fopen(anPtr->logfname, "w");
	if (anPtr->logfp != NULL) {
	  fprintf(anPtr->logfp, "# Log-File Anzeige %s\n", argv[0]);
	  if ( (anPtr->vname != NULL) && (strlen(anPtr->vname)) ) {
	    fprintf(anPtr->logfp, "#col: t %s\n", anPtr->vname);
	  }
	  anPtr->dolog = 1;
	  StartLog(clientData);
	} else {
	  free(anPtr->logfname);
	}
      }
    } else if ( (strcmp(argv[2], "off") == 0)
		|| (strcmp(argv[2], "no") == 0)
		|| (strcmp(argv[2], "0") == 0)
		|| (strcmp(argv[2], "false") == 0) ) {
      Tk_DeleteTimerHandler(anPtr->timer);
      if (anPtr->dolog)
	fclose(anPtr->logfp);
      anPtr->dolog = 0;
    }
  }
  
  Tk_Release((ClientData) anPtr);
  
  return(TCL_OK);
}

/*********************************************************
 * neues Canvas fuer Graphic erzeugen,initialisern und
 * mit Tcl/Tk verbinden
 */

int Tk_AnalogWidgetCreate ( ClientData clientData , Tcl_Interp *interp ,
			   int argc , char *argv[] )
{
  Tk_WAnalog *anPtr;
  Tk_Window tkwin;
  Tk_Window main;

  main = (Tk_Window) clientData;
    
  if (argc < 2) {
    Tcl_AppendResult(interp, "wrong # args: shold be \"",
		     argv[0], " pathName ?option?\"",
		     (char *) NULL);
    return(TCL_ERROR);
  }

  /* neues Fenster erzeugen */
  
  tkwin = Tk_CreateWindowFromPath (interp, main, argv[1], (char *) NULL);
  
  if (tkwin == NULL)
    return(TCL_ERROR);
  
  Tk_SetClass(tkwin, "Tk_WAnalog");
  
  /* Speicher fuer neues Grafikwidget allozieren */
  
  anPtr = (Tk_WAnalog *) malloc(sizeof(Tk_WAnalog));

  /* initialisieren */
  
  anPtr->tkwin = tkwin;
  anPtr->display = Tk_Display(tkwin);
  anPtr->interp = interp;

  anPtr->width = 0;
  anPtr->height = 0;
  anPtr->w = anPtr->h = 0;
  anPtr->linewidth = 1;

  anPtr->fgcolor = 0;
  anPtr->bgcolor = 0;
  anPtr->bgborder = anPtr->fgborder = NULL;

  anPtr->relief = TK_RELIEF_SUNKEN;

  anPtr->scale = 0;

  anPtr->gc = None;
  anPtr->pix = None;
  anPtr->outpix = 0;

  anPtr->updatePending = 0;

  anPtr->vname = NULL;
  anPtr->value = 0.0;

  anPtr->logfname = NULL;
  anPtr->dolog = 0;

  anPtr->text = NULL;
  anPtr->dotxt = 0;

  anPtr->cmdamax = anPtr->cmdamin = NULL;
  anPtr->cmdreset = NULL;
  anPtr->setamin = anPtr->setamax = 0;
  anPtr->amin = anPtr->amax = 0.0;
  anPtr->alarmflag = anPtr->resetflag = 0;

  /* Register a handler for when the window is exposed or resized */

  Tk_CreateEventHandler(anPtr->tkwin,
			ExposureMask | StructureNotifyMask,
			Tk_AnalogEventProc, (ClientData) anPtr);
  
  /* Create a Tcl command that operates on the widget */
  
  Tcl_CreateCommand(interp, Tk_PathName(anPtr->tkwin),
		    Tk_AnalogWidgetCmd, (ClientData) anPtr,
		    (Tcl_CmdDeleteProc *) NULL);
  
  /* Parse the command line arguments */

  if (Tk_AnalogConfigure(interp, anPtr, argc-2, argv+2, 0) != TCL_OK) {
    Tk_DestroyWindow(anPtr->tkwin);
    return(TCL_ERROR);
  }

  interp->result = Tk_PathName(anPtr->tkwin);

  Tk_DoWhenIdle(Tk_AnalogDisplay, (ClientData) anPtr);
  anPtr->updatePending = 1;

  return(TCL_OK);
}

/*********************************************************
 * Resourcen fuer Widget allozieren
 */

static int Tk_AnalogConfigure ( Tcl_Interp *interp , Tk_WAnalog *anPtr ,
			       int argc , char *argv[] , int flags )
{
  Tk_Window tkwin = anPtr->tkwin;
  Display *display = Tk_Display(tkwin);
  XGCValues gcValues;
  int lww, lwh;
  char *value;
  char *endptr;

  if (Tk_ConfigureWidget(interp, anPtr->tkwin, configSpecs, argc, argv,
			 (char *) anPtr, flags) != TCL_OK) {
    return(TCL_ERROR);
  }

  anPtr->fgcolor = Tk_3DBorderColor(anPtr->fgborder)->pixel;
  anPtr->bgcolor = Tk_3DBorderColor(anPtr->bgborder)->pixel;
#if ( (TK_MAJOR_VERSION  == 3) )
  anPtr->wcolor = Tk_GetColor(interp, anPtr->tkwin, Tk_Colormap(anPtr->tkwin),
			      Tk_GetUid("red"))->pixel;
#else
  anPtr->wcolor = Tk_GetColor(interp, anPtr->tkwin, Tk_GetUid("red"))->pixel;
#endif

  if (anPtr->width == 0) {
    if (anPtr->height == 0) {
      anPtr->width = 160;
      anPtr->height = 120;
    } else {
      anPtr->width = div(4 * anPtr->height, 3).quot;
    }
  } else {
    if (anPtr->height == 0)
      anPtr->height = div(3 * anPtr->width, 4).quot;
  }

  Tk_GeometryRequest(tkwin, anPtr->width, anPtr->height);
  Tk_SetInternalBorder (tkwin, anPtr->borderwidth);
  lww = div(anPtr->width, 160).quot;
  lwh = div(anPtr->height, 120).quot;
  anPtr->linewidth = (lwh > lww) ? lww : lwh;

  anPtr->ox1 = 0;
  anPtr->ox2 = 0;
  anPtr->oy1 = 0;
  anPtr->oy2 = 0;

  Tk_SetWindowBackground(tkwin, anPtr->bgcolor);

  /* Free Grafikcontext */
  
  if (anPtr->gc != None) 
    Tk_FreeGC(anPtr->display, anPtr->gc);
  
  /* Grafikkontext fuer Widget erstellen */
  
  gcValues.line_width = anPtr->linewidth;
  gcValues.line_style = LineSolid;
  gcValues.foreground = anPtr->fgcolor;
  gcValues.background = anPtr->bgcolor;
  gcValues.function = GXcopy; 
  gcValues.graphics_exposures = False;
  
  anPtr->gc = Tk_GetGC(tkwin,
		       GCForeground | GCBackground | GCLineWidth | GCLineStyle
		       | GCFunction | GCGraphicsExposures,
		       &gcValues);


  /* pixmap fuer Grafik erstellen */
  
#if ( (TK_MAJOR_VERSION  == 3) )
  if (anPtr->pix != None)
    XFreePixmap(anPtr->display, anPtr->pix);

  anPtr->pix = XCreatePixmap(display, RootWindowOfScreen(Tk_Screen(anPtr->tkwin)), 
			     anPtr->width, anPtr->height,
			     Tk_Depth(tkwin) );
#else
  if (anPtr->pix != None)
    Tk_FreePixmap(anPtr->display, anPtr->pix);

  anPtr->pix = Tk_GetPixmap(display, RootWindowOfScreen(Tk_Screen(anPtr->tkwin)), 
			    anPtr->width, anPtr->height,
			    Tk_Depth(tkwin) );
#endif
  
  /* Anzeige-Scale aufbauen */
  
  plotAnalogScale(anPtr, interp);
  
  /* Tcl-Variable zuordnen */
  
  /* Eliminate any existing trace on variables monitored by the button. */
  
  if ( (anPtr->vname != NULL) && (strlen(anPtr->vname)) ) {
    Tcl_UntraceVar(interp, anPtr->vname, 
		   TCL_GLOBAL_ONLY | TCL_TRACE_WRITES | TCL_TRACE_UNSETS,
		   AnalogTextVarProc, (ClientData) anPtr);

    /* The button must display the value of a variable: set up a trace
     * on the variable's value, create the variable if it doesn't
     * exist, and fetch its current value.
     */

    value = Tcl_GetVar(interp, anPtr->vname, TCL_GLOBAL_ONLY); 
    if (value == NULL) {
      if (Tcl_SetVar(interp, anPtr->vname, "0.0",
		     TCL_GLOBAL_ONLY|TCL_LEAVE_ERR_MSG) == NULL) {
	return TCL_ERROR;
      }
      anPtr->value = 0.0;
    } else {
      anPtr->value = strtod(value, &endptr);
    }
    Tcl_TraceVar(interp, anPtr->vname,
		 TCL_GLOBAL_ONLY | TCL_TRACE_WRITES | TCL_TRACE_UNSETS,
		 AnalogTextVarProc, (ClientData) anPtr);
    
  } else {
    Tcl_AppendResult(interp, 
	 "need a variable : shold be \"analog ?.w? -variable ?value? ?arg arg ...?\"",
		     (char *) NULL);
    
    return(TCL_ERROR);
  }

  if (!anPtr->updatePending) {
    Tk_DoWhenIdle(Tk_AnalogDisplay, (ClientData) anPtr);
    anPtr->updatePending = 1;
  }

  return(TCL_OK);
}
 
/*********************************************************
 * Pixmap auf Bildschirm ausgeben
 */

static void Tk_AnalogDisplay ( ClientData clientData )
{
  Tk_WAnalog *anPtr = (Tk_WAnalog *) clientData;
  Tk_Window tkwin = anPtr->tkwin;
  int px, py;

  anPtr->updatePending = 0;

  if (!Tk_IsMapped(tkwin)) { 
    return;
  }

  px = div(Tk_Width(anPtr->tkwin) - anPtr->width, 2).quot;
  py = div(Tk_Height(anPtr->tkwin) - anPtr->height, 2).quot;

  if (!anPtr->outpix)
    plotScale(anPtr, px, py);

  plotZeiger(anPtr, px, py);
}

/*********************************************************
 * Zeiger auf Bildschirm ausgeben mit Offset (px,py)
 */

static void plotScale ( Tk_WAnalog *anPtr , int px , int py )
{
  if (anPtr->pix != None) {    
    XCopyArea(Tk_Display(anPtr->tkwin), anPtr->pix, Tk_WindowId(anPtr->tkwin), 
	      anPtr->gc, 0, 0, (unsigned int) anPtr->width,
	      (unsigned int) anPtr->height, px, py);
    anPtr->outpix = 1;
  }
}

/*********************************************************
 */

static int mrint ( double nmbr )
{
  int ret = 0;

  ret = (int) floor(nmbr);
  if (nmbr - floor(nmbr) > 0.5)
    ret += 1;

  return ret;
}

/*********************************************************
 * Zeiger auf Bildschirm ausgeben mit Offset (px,py)
 */

static void plotZeiger ( Tk_WAnalog *anPtr , int px , int py )
{
  Display *display = Tk_Display(anPtr->tkwin);
  int px1, py1, px2, py2;
  int bd;
  double w;
  double xin;
  int hh, ww;

  /* extrema beruecksichtigen */

  xin = anPtr->value;
  if (anPtr->value >= anPtr->min) {
    if (anPtr->value > anPtr->max) {
      xin = anPtr->max;
    }
  } else {
    xin = anPtr->min;
  }

  w = anPtr->m * xin + anPtr->n;
  px1 = anPtr->midx + mrint(anPtr->zglen * cos(w)) + px;
  py1 = anPtr->midy - mrint(anPtr->zglen * sin(w)) + py;

  px2 = mrint(((double) (anPtr->midy - anPtr->zl)) / tan(w));
  px2 += anPtr->midx + px;
  py2 = anPtr->zl + py;

  if (anPtr->oy1 != anPtr->oy2) {
    XSetForeground(display, anPtr->gc, anPtr->bgcolor);  
    XDrawLine(display, Tk_WindowId(anPtr->tkwin), anPtr->gc,
	      anPtr->ox1, anPtr->oy1, anPtr->ox2, anPtr->oy2);
  }

  anPtr->ox1 = px1;
  anPtr->ox2 = px2;
  anPtr->oy1 = py1;
  anPtr->oy2 = py2;

  XSetForeground(display, anPtr->gc, anPtr->fgcolor);  

  if ( (anPtr->dotxt) && (anPtr->text != NULL) ) {
    XDrawImageString(display, Tk_WindowId(anPtr->tkwin), anPtr->gc,
		     anPtr->tx+px, anPtr->ty+py, anPtr->text, 
		     (int) strlen(anPtr->text));
  }


  bd = anPtr->borderwidth + anPtr->linewidth;
  hh = anPtr->h + bd + py;
  ww = anPtr->w + bd + px;
  XDrawLine(display, Tk_WindowId(anPtr->tkwin), anPtr->gc,
		 bd+px, hh, ww, hh);
	    /* anPtr->h+bd+py, anPtr->w+bd+px, anPtr->h+bd+py); */

  XDrawLine(display, Tk_WindowId(anPtr->tkwin), anPtr->gc,
	    px1, py1, px2, py2);

}

/*********************************************************
 * Events, die das Graphic-Widget betreffen, abhandeln
 */

static void Tk_AnalogEventProc ( ClientData clientData , XEvent *eventPtr )
{
  Tk_WAnalog *anPtr = (Tk_WAnalog *) clientData;
  int px, py;

  if ( (eventPtr->type == Expose) || (eventPtr->type == ConfigureNotify) ) {
    if (!anPtr->updatePending) {
      if (Tk_IsMapped(anPtr->tkwin)) { 
	px = div(Tk_Width(anPtr->tkwin) - anPtr->width, 2).quot;
	py = div(Tk_Height(anPtr->tkwin) - anPtr->height, 2).quot;
	plotScale(anPtr, px, py);
      }
      Tk_DoWhenIdle(Tk_AnalogDisplay, (ClientData) anPtr);
      anPtr->updatePending = 1;
    }
  } else  if (eventPtr->type == DestroyNotify) {
    Tcl_DeleteCommand(anPtr->interp, Tk_PathName(anPtr->tkwin));

    anPtr->tkwin = NULL;

    if (anPtr->updatePending) 
      Tk_CancelIdleCall(Tk_AnalogDisplay, (ClientData) anPtr);
    
    Tk_EventuallyFree((ClientData) anPtr, (Tcl_FreeProc *) Tk_AnalogWidgetDestroy);
  } 
}

/*********************************************************
 * Canvas mit allem Drum und Dran frei geben
 */

static void Tk_AnalogWidgetDestroy ( ClientData clientData )
{
  Tk_WAnalog *anPtr = (Tk_WAnalog *) clientData;

  if (anPtr->vname != NULL) {
    Tcl_UntraceVar(anPtr->interp, anPtr->vname,
		   TCL_GLOBAL_ONLY | TCL_TRACE_WRITES | TCL_TRACE_UNSETS,
		   AnalogTextVarProc, (ClientData) anPtr);
  }

  if (anPtr->cmdamin != NULL)
    free(anPtr->cmdamin);
  if (anPtr->cmdamax != NULL)
    free(anPtr->cmdamax);
  if (anPtr->cmdreset != NULL)
    free(anPtr->cmdreset);

  if (anPtr->dolog) {
    Tk_DeleteTimerHandler(anPtr->timer);
    fclose(anPtr->logfp);
    free(anPtr->logfname);
  }

  /* Free Widget Configuration Resource */

  Tk_FreeOptions(configSpecs, (char *) anPtr, anPtr->display, 0);

  /* Free Pixmap */

  if (anPtr->pix != None) {
#if ( (TK_MAJOR_VERSION  == 3) )
    XFreePixmap(anPtr->display, anPtr->pix);
#else
    Tk_FreePixmap(anPtr->display, anPtr->pix);
#endif
  }

  /* Free Grafikcontext */
  
  if (anPtr->gc != None) 
    Tk_FreeGC(anPtr->display, anPtr->gc);
  
  free(anPtr);	/* Free struct Tk_WAnalog */
}

/*********************************************************
 * Ausgabe eines Spectrum in das Widget
 */

static char *AnalogTextVarProc( ClientData clientData , Tcl_Interp *interp , 
				char *name1 , char *name2 , int flags )
{
  Tk_WAnalog *anPtr = (Tk_WAnalog *) clientData;
  Display *display = Tk_Display(anPtr->tkwin);
  char *value;
  double dbl;
  int px, py;
  Pixel wmaxcolor, wmincolor;
  
  /*
   * If the variable is unset, then immediately recreate it unless
   * the whole interpreter is going away.
   */
  
  if (flags & TCL_TRACE_UNSETS) {
    if ((flags & TCL_TRACE_DESTROYED) && !(flags & TCL_INTERP_DESTROYED)) {
      Tcl_SetVar(interp, anPtr->vname, "0.0", TCL_GLOBAL_ONLY);
      Tcl_TraceVar(interp, anPtr->vname,
		   TCL_GLOBAL_ONLY|TCL_TRACE_WRITES|TCL_TRACE_UNSETS,
		   AnalogTextVarProc, clientData);
    }
    return (char *) NULL;
  }
  
  value = Tcl_GetVar(interp, anPtr->vname, TCL_GLOBAL_ONLY);
  if (value == NULL) {
    value = "";
  }
  if (Tcl_GetDouble(interp, value, &dbl) == TCL_OK) {
    anPtr->value = dbl;
  } else {
    anPtr->value = anPtr->min;
    value = NULL;
  }
  
  /* Warnlampe fuer Extrema setzten */

  wmincolor = anPtr->bgcolor;
  wmaxcolor = anPtr->bgcolor;
  if (anPtr->value >= anPtr->min) {
    if (anPtr->value > anPtr->max) {
      wmaxcolor = anPtr->wcolor;
    }
  } else {
    wmincolor = anPtr->wcolor;
  }

  /* Zeiger ausgeben */

  if (Tk_IsMapped(anPtr->tkwin)) { 
    px = div(Tk_Width(anPtr->tkwin) - anPtr->width, 2).quot;
    py = div(Tk_Height(anPtr->tkwin) - anPtr->height, 2).quot;
    
    if ( (!anPtr->outpix) && (!anPtr->updatePending) ) {
      plotScale(anPtr, px, py);
      anPtr->outpix = 1;
    }

    XSetForeground(display, anPtr->gc, wmincolor);  
    XFillRectangle(display, Tk_WindowId(anPtr->tkwin), anPtr->gc,
		   anPtr->wxmin+px, anPtr->wy+py, anPtr->wd, anPtr->wd);
    XSetForeground(display, anPtr->gc, wmaxcolor);  
    XFillRectangle(display, Tk_WindowId(anPtr->tkwin), anPtr->gc,
		   anPtr->wxmax+px, anPtr->wy+py, anPtr->wd, anPtr->wd);

    plotZeiger(anPtr, px, py);
  }

  /* Alarm ausfuehren oder reseten */

  if ( (anPtr->setamin) && (anPtr->value < anPtr->amin) ) {
    anPtr->resetflag = 0;
    if ( (!anPtr->alarmflag) && (anPtr->cmdamin != NULL) ) {
      DoAlarmCommand(interp, anPtr->cmdamin);
      anPtr->alarmflag = 1;
    }
  } else if ( (anPtr->setamax) && (anPtr->value > anPtr->amax) ) {
    anPtr->resetflag = 0;
    if ( (!anPtr->alarmflag) &&(anPtr->cmdamax != NULL) ) {
      DoAlarmCommand(interp, anPtr->cmdamax);
      anPtr->alarmflag = 1;
    }
  } else if ( (anPtr->setamin) || (anPtr->setamax) ) {
    anPtr->alarmflag = 0;
    if ( (!anPtr->resetflag) && (anPtr->cmdreset != NULL) ) {
      DoAlarmCommand(interp, anPtr->cmdreset);
      anPtr->resetflag = 1;
    }
  }
  
  return (char *) NULL;
}

/*********************************************************
 * Starten eines log (d.h. Timer setzten)
 */

static void StartLog ( ClientData clientData )
{
  register Tk_WAnalog *anPtr = (Tk_WAnalog *) clientData;

  anPtr->startime = time(NULL) + 2;
  anPtr->timer = Tk_CreateTimerHandler(2000, DoLog, (ClientData) anPtr);
}

/*********************************************************
 * log ausfuehren
 */

static void DoLog ( ClientData clientData )
{
  register Tk_WAnalog *anPtr = (Tk_WAnalog *) clientData;
  time_t now;

  Tk_DeleteTimerHandler(anPtr->timer);

  now = time(NULL) - anPtr->startime;
  fprintf(anPtr->logfp, "%ld %g\n", now, anPtr->value);
  fflush(anPtr->logfp);
  
  anPtr->timer = Tk_CreateTimerHandler(995, DoLog, (ClientData) anPtr);
}

/*********************************************************
 * evaluate an script on alarm
 * return : TCL_OK    - success
 *          TCL_ERROR - error in script
 */

static int DoAlarmCommand ( Tcl_Interp *interp , char *script )
{
    Tcl_DString buffer;
    int code;

    if (strlen(script)) {
      Tcl_DStringInit(&buffer);
      Tcl_DStringAppend(&buffer, script, -1);
      code = Tcl_GlobalEval(interp, Tcl_DStringValue(&buffer));
      Tcl_DStringFree(&buffer);
    } else {
      code = TCL_OK;
    }

    return code;
}

/*********************************************************
 * Ausgabe der Anzeige-Skale auf die Pixmap
 */

static int plotAnalogScale ( Tk_WAnalog *anPtr , Tcl_Interp *interp )
{
  Display *display = Tk_Display(anPtr->tkwin);
  int result = TCL_OK;
  int bd;
  double dbl;
  double alpha, beta;
  int dzx, dzy;
#ifdef PLOT_ANA_TEST
  int zmin, zmax, zhigh;
#endif
  int smin;
  double sh, sb, db;
  char anzeigetxt[100];
  Pixel green;
  XPoint gbar[4];
  double amin, amax;
  int px1, px2, py1, py2;

  anPtr->outpix = 0;

  /* Pixmap initialisieren */

  XSetForeground(display, anPtr->gc, anPtr->fgcolor);  
#if ( (TK_MAJOR_VERSION  == 3) )
  Tk_Fill3DRectangle(display, anPtr->pix, anPtr->bgborder, 
		     0, 0, anPtr->width, anPtr->height,
		     anPtr->borderwidth, anPtr->relief); 
#else
  Tk_Fill3DRectangle(anPtr->tkwin, anPtr->pix, anPtr->bgborder, 
		     0, 0, anPtr->width, anPtr->height,
		     anPtr->borderwidth, anPtr->relief); 
#endif

  bd = anPtr->borderwidth + anPtr->linewidth;
  anPtr->w = anPtr->width - 2 * bd - 1;
  anPtr->h = anPtr->height - 2 * bd - 1;

  XSetForeground(display, anPtr->gc, anPtr->fgcolor);
  XDrawRectangle(display, anPtr->pix, anPtr->gc,
		 bd, bd, anPtr->w, anPtr->h);

  /* Zeiger definieren */

  XSetForeground(display, anPtr->gc, anPtr->fgcolor);  

  /* Nullpunkt des Pixelkoord.-Systems in der linken/oberen Ecke */
  /* Ausschlag zeiger links/rechts */
  dzx = div((3 * ((int) anPtr->w)), 8).quot;
  /* Hoehe des Zeigers bei min/max Auschlag */
  dzy = div((3 * ((int) anPtr->h)), 4).quot;
  /* Hohe des min/max-Auschlages von oberer Kante */
  smin = div(2 * ((int) anPtr->h), 3).quot;

  /* Mittelpunkt des Zeiger um den er auschlaegt in Pixelkoord */
  anPtr->midx = div(anPtr->width, 2).quot;
  anPtr->midy = smin + dzy;
  /* laenge des Zeigers */
  dbl = ((double) (dzx));
  anPtr->zglen = dbl * dbl;
  dbl = (double) (dzy);
  anPtr->zglen += (dbl * dbl);
  anPtr->zglen = sqrt(anPtr->zglen);
  /* untere Grenze fuer Sichtbarkeit des Zeigers in Pixelkoord. */
  anPtr->zl = anPtr->height - bd - anPtr->linewidth;

#ifdef PLOT_ANA_TEST
  /* maximale hoehe des Zeigerausschlages in Pixelkoord. */
  zhigh = anPtr->midy - anPtr->zglen;
  /* Pixelkoordinaten des min/max Auschlag des Zeigers */
  zmin = anPtr->midx - dzx;
  zmax = anPtr->midx + dzx;

  XDrawLine(display, anPtr->pix, anPtr->gc,
	    zmin, zhigh, zmax, zhigh);
  
  XDrawLine(display, anPtr->pix, anPtr->gc,
	    zmin, smin, zmax, smin);
  
  XDrawLine(display, anPtr->pix, anPtr->gc,
	    zmin, zhigh, zmin, smin);
  
  XDrawLine(display, anPtr->pix, anPtr->gc,
	    zmax, zhigh, zmax, smin);
  
  XDrawLine(display, anPtr->pix, anPtr->gc,
	    anPtr->midx, anPtr->zl, zmin, smin);
  
  XDrawLine(display, anPtr->pix, anPtr->gc,
	    anPtr->midx, anPtr->zl, zmax, smin);
#endif

  /* Geradenanpassung zur Umrechnung Eingabewert -> Anzeigewinkel */
  alpha = atan(((double) dzy) / ((double) dzx));
  beta = M_PI - alpha;
  anPtr->m = (alpha - beta) / (anPtr->max - anPtr->min);
  anPtr->n = (alpha + beta - anPtr->m *(anPtr->max + anPtr->min)) / 2.0;

  /* Parameter fuer Warnlampe setzten */

  anPtr->wy = div((((int) anPtr->h) - smin), 2).quot + smin;
  anPtr->wxmin = anPtr->midx - dzx;
  anPtr->wxmax = anPtr->midx + dzx;
  anPtr->wd = 4 * anPtr->linewidth;

  /* Laenge eines Scalestriches */

  sh = ((double) anPtr->h) / 6.0;
  if (sh > (((double) anPtr->w) / 12.0) )
    sh = ((double) anPtr->w) / 12.0;

  /* Hintergrund Alarmbereich ausgeben */

  if ( (anPtr->setamax) || (anPtr->setamin) ) {
#if ( (TK_MAJOR_VERSION  == 3) )
    green = Tk_GetColor(interp, anPtr->tkwin, Tk_Colormap(anPtr->tkwin), 
			Tk_GetUid("green"))->pixel;
#else
    green = Tk_GetColor(interp, anPtr->tkwin, Tk_GetUid("green"))->pixel;
#endif
    XSetForeground(display, anPtr->gc, green);
    amin = anPtr->amin;
    if (!anPtr->setamin)
      amin = anPtr->min;
    if (amin < anPtr->min)
      amin = anPtr->min;
    amax = anPtr->amax;
    if (!anPtr->setamax)
      amax = anPtr->max;
    if (amax > anPtr->max)
      amax = anPtr->max;
    sb = 1.1 * sh;
    db = fabs(amax - amin);
    db /= ((double) anPtr->w);
    
    if (amin <= amax) {
      for (dbl = amin; (dbl <= amax); dbl += db) {
	if (plotScaleLine(anPtr, dbl, sb, 1, &px1, &py1, &px2, &py2)) {
	  gbar[0].x = px1;
	  gbar[0].y = py1;
	  gbar[3].x = px2;
	  gbar[3].y = py2;
	  if (dbl+db <= amax) {
	    if (plotScaleLine(anPtr, dbl+db, sb, 1, &px1, &py1, &px2, &py2)) {
	      gbar[1].x = px1;
	      gbar[1].y = py1;
	      gbar[2].x = px2;
	      gbar[2].y = py2;
	      XFillPolygon(display, anPtr->pix, anPtr->gc,
			   gbar, 4, Nonconvex, CoordModeOrigin);
	    }
	  }
	}
      }
    }
  }

  /* Scala ausgeben */

  doscale(anPtr, sh, anzeigetxt);

  /* Text an die Stelle (tx,ty)*/

  if ( (anPtr->text != NULL) && (strlen(anPtr->text)) )
    strcpy(anzeigetxt, anPtr->text);
  else
    strcpy(anzeigetxt, "");

  if (strlen(anzeigetxt)) {
    anPtr->ty = abs(anPtr->zl - anPtr->midy + mrint(anPtr->zglen));
    anPtr->ty = anPtr->zl - div(anPtr->ty, 2).quot;
    anPtr->tx = anPtr->midx - div(9 * ((int) strlen(anzeigetxt)), 2).quot;
    anPtr->dotxt = 1;

    XDrawImageString(display, anPtr->pix, anPtr->gc,
		     anPtr->tx, anPtr->ty, anzeigetxt,
		     (int) strlen(anzeigetxt));
  }

  return(result);
}

/*********************************************************
 */

static double mpow10 ( int nmbr )
{
  double dbl = 0.0;

  dbl = pow(10.0, (double) nmbr);

  return dbl;
}

/*********************************************************
 */

static void doscale (  Tk_WAnalog *anPtr , double sklen , char *txt )
{
  Display *display = Tk_Display(anPtr->tkwin);
  int i;
  int base;
  double pbase;
  int zl, zh;
  double smin, smax;
  double fbase;
  double fmin, fmax;
  double ds;
  double sl, sh;
  double sk[100];
  char skt[10][20];
  int skl, skh;
  int px1, px2, py1, py2;
  
  /* groesste Zahl auf Skala finden */
  ds = fabs(anPtr->max);
  if (ds < fabs(anPtr->min)) {
    ds = fabs(anPtr->min);
  }
  fbase = floor(log10(ds));
  
  base = 3 * ((int) floor(fbase / 3.0));
  pbase = mpow10(base);
  
  /* minimum und maximum der scale berechnen */
  
  smin = anPtr->min / pbase;
  smax = anPtr->max / pbase;
  
  if ( ( (base > 0) && (base % 3) == 0)
       && ( ((fabs(smax) >= 1.0) && (fabs(smax) <= 1.05)
	     && (fabs(smin) < 1.0)) ||
	    ((fabs(smin) >= 1.0) && (fabs(smax) <= 1.05)
	     && (fabs(smax) < 1.0)) ) ) {
    smin *= 1000;
    smax *= 1000;
    base -= 3;
    pbase = mpow10(base);
  }
  
  fmax = (smax - smin);
  fmin = 0;
  
  fbase = 0.001;
  if (fmax >= 0.01)
    fbase = 0.01;
  if (fmax >= 0.1)
    fbase = 0.1;
  if (fmax >= 1.0)
    fbase = 1.0;
  if (fmax >= 10.0)
    fbase = 10.0;
  if (fmax >= 100.0)
    fbase = 100.0;
  if (fmax >= 1000.0)
    fbase = 1000.0;
  fmax /= fbase;
  
  /* Skaleneinteilung definieren (sl-lange, sh-halbe Striche) */
  sl = 0.2;
  sh = 0.1;
  ds = fabs(fmax);
  if (ds > 9.0) {
    sl = 2.0;
    sh = -1;
  } else if (ds > 6.0) {
    sl = 2.0;
    sh = 1.0;
  } else if (ds > 3.0) {
    sl = 1.0;
    sh = 0.5;
  } else if (ds > 1.6) {
    sl = 0.5;
    sh = 0.25;
  }
  
  sl *= fbase;
  sh *= fbase;
  fmin = anPtr->min / pbase;
  fmax =  anPtr->max / pbase;
  
  /* lange Skalenstriche mit text berechnen */
  
  zl = (int) floor(fmin / sl);
  if ((((double) zl) * sl) < fmin)
    zl++;
  
  zh = (int) floor(fmax / sl);
  if ((((double) zh) * sl) > fmax)
    zh--;
  
  skl = 0;
  ds = sl * pbase;
  for (i = zl; ((i <= zh) && (skl < 10)); i++) {
    sk[skl] = ((double) i) * ds;
    sprintf(skt[skl], "%g", ((double) i) * sl);
    skl++;
  }
  
  /* halblange Skalenstriche ohne text berechnen */
  
  if (sh > 0.0) {
    zl = (int) floor(fmin / sh);
    if ((((double) zl) * sh) < fmin)
      zl++;
    
    zh = (int) floor(fmax / sh);
    if ((((double) zh) * sh) > fmax)
      zh--;
    
    skh = 0;
    ds = sh * pbase;
    for (i = zl; ((i <= zh) && (skh < 90)); i++) {
      sk[skh+10] = ((double) i) * ds;
      skh++;
    }
    
    /* "doppelte Skalenstrich entfernen */
    
    zl = zh = 0;
    for (i = 0; ( (i < skl) && (skh > 0) ); i++) {
      while( (sk[zh+10] < sk[i]) && (zh < skh) )
	zh++;
      if (sk[i] == sk[zh+10]) {
	for (zl = zh; (zl < skh-1); zl++)
	  sk[zl+10] = sk[zl+11];
	skh--;
      }
    }
    
  } else {
    skh = 0;
  }

  if (base) {
    strcpy(txt, "");
    if (base == 1)
      sprintf(txt, "x10");
    else if (base == 1)
      sprintf(txt, "x100");
    else {
      sprintf(txt, "x10^%d", base);
    }
  }
  /* Skala ausgeben */
  
  XSetForeground(display, anPtr->gc, anPtr->fgcolor);  

  XSetFont(display, anPtr->gc, XLoadFont(display, "9x15"));

  for (i = 0; (i < skl); i++) {

    if (plotScaleLine(anPtr, sk[i], sklen, 1, &px1, &py1, &px2, &py2)) {
      XDrawLine(display, anPtr->pix, anPtr->gc, px1, py1, px2, py2);
      XDrawImageString(display, anPtr->pix, anPtr->gc,
		       px2 - div((int) (9 * strlen(skt[i])), 2).quot, 
		       py2 - div(abs(py1 - py2), 2).quot, 
		       skt[i], (int) strlen(skt[i]));
    }
  }
  for (i = 0; (i < skh); i++) {
    if (plotScaleLine(anPtr, sk[i+10], sklen, 2, &px1, &py1, &px2, &py2)) {
      XDrawLine(display, anPtr->pix, anPtr->gc, px1, py1, px2, py2);
    }
  }
}

/*********************************************************
 * Berechnung eines Scalenstriches fuer einen Wert xin
 * return : 1 - success
 *          0 - failed (xin out of scala)
 */

static int plotScaleLine ( Tk_WAnalog *anPtr , double xin , 
			   double len , int part,
			   int *px1 , int *py1 , int *px2 , int *py2 )
{
  int px, py;
  double w;
  double sl;
  double plen = 1.0;
  int fhigh;

  if (part == 2) {
    plen = 2.0;
  }

  /* falls xin nicht in der Scala liegt return */
  if (xin >= anPtr->min) {
    if (xin > anPtr->max)
      return(0);
  } else {
    return(0);
  }

  /* Winkel des Zeigers berechnen */
  w = anPtr->m * xin + anPtr->n;
  /* Endpunkt des Zeiger in Pixelkoordinaten brechnen */
  px = anPtr->midx + mrint(anPtr->zglen * cos(w));
  py = anPtr->midy - mrint(anPtr->zglen * sin(w));
  
  sl = len / 5.0;
  *px1 = px + mrint(sl * cos(w));
  *py1 = py - mrint(sl * sin(w));
  fhigh = abs(*py1 - py);
  
  sl *= 6.0;
  sl /= plen;
  *px2 = px + mrint(sl * cos(w));
  *py2 = py - mrint(sl * sin(w));
  
  return(1);
}

/* ******************************************************* */
/* File: tkanalog.c                                      */
/* ******************************************************* */
