/* ******************************************************* */
/*
 * File: tkanalogPkgInit.c
 *
 * Author: Martin Hachenberg
 *
 * Initialisiert die Tcl-tkanalog Erweiterung.
 *
 * Copyright (C) 1998, 1999 Universitaet zu Koeln, Germany.
 * Written by Martin Hachenberg (martinh@ikp.uni-koeln.de).
 * 
 * This file is part of the tkanalog-package.
 * 
 * tkanalog is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 * 
 * tkanalog is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with tkanalog -- see the file COPYING.  If not, write to the Free
 * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA.
 */
/* ******************************************************* */

#include "tkanalog.h"

/*
 * Im Folgenden werden die neuen Tcl-Befehlsnamen und die C-Funktionen,
 * die sie ausfuehren festgelegt.
 */

typedef struct {
    char *name;			/* Name des Befehls. */
    Tcl_CmdProc *cmdProc;	/* C-Befehlsprozedur. */
} TclCmd;

/* tcl und C Namen verknuepfen */
static TclCmd commands[] = {
    {"analog",	Tk_AnalogWidgetCreate},
    {(char *) NULL,	(Tcl_CmdProc *) NULL}
};

/* ******************************************************* */
/*
 * Tkanalog_Init --
 *
 *	Diese Prozedur baut saemtliche tkanalog-Befehle in den benutzten 
 *      Tcl-Interpreter ein.
 *
 * Folgen:
 *
 *	Liefert einen Tcl_Standartrueckgabewert und setzt interp->result
 *	wenn ein Fehler auftritt.
 */
/* ******************************************************* */

int Tkanalog_Init ( Tcl_Interp *interp )
{
    TclCmd *cmdPtr;

    if (Tcl_PkgRequire(interp, "Tcl", TCL_VERSION, 1) == NULL) {
	return TCL_ERROR;
    }

    Tcl_SetVar(interp, "tkanalog_patchLevel",
	       TKANALOG_PATCH_LEVEL, TCL_GLOBAL_ONLY);
    Tcl_SetVar(interp, "tkanalog_version", 
	       TKANALOG_VERSION, TCL_GLOBAL_ONLY);

    if (Tcl_PkgProvide(interp, "tkanalog", TKANALOG_VERSION) != TCL_OK) {
	return TCL_ERROR;
    }

    for (cmdPtr = commands; cmdPtr->name != NULL; cmdPtr++) {
      Tcl_CreateCommand(interp, cmdPtr->name, cmdPtr->cmdProc,
			(ClientData) Tk_MainWindow(interp),
			(Tcl_CmdDeleteProc *) NULL);
    }

    return TCL_OK;
}

/* ******************************************************* */
/* File: tkanalogPkgInit.c                                 */
/* ******************************************************* */




















