/* ******************************************************* */
/*
 * File: tclAnalogAppInit.c
 *
 * Author: Martin Hachenberg
 *
 * Erzeugung einer neuen 'wish' mit dem Widgets analog
 * fuer Tcl 7.4 / Tk 4.0
 *
 * Copyright (C) 1998, 1999 Universitaet zu Koeln, Germany.
 * Written by Martin Hachenberg (martinh@ikp.uni-koeln.de).
 * 
 * This file is part of the tkanalog-package.
 * 
 * tkanalog is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 * 
 * tkanalog is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with tkanalog -- see the file COPYING.  If not, write to the Free
 * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA.
 */
/* ******************************************************* */

#define _tclAnalogAppInit_ 1

#include "tcl.h"
#include "tk.h"

/* external variables */
/* none */

/* global variables */

/* none */

/* local variables */
/* none */

/* external functions */

extern int Tk_AnalogWidgetCreate ( ClientData clientData, Tcl_Interp *interp , 
				   int argc , char *argv[] );

/* global functions */

int Tcl_AppInit ( Tcl_Interp *interp );

/* local functions */

static int Analog_Init ( Tcl_Interp * );

/* ******************************************************* */

int main ( int argc, char *argv[] )
{
  Tk_Main(argc, argv, Tcl_AppInit);
  return(0);
}

/* ******************************************************* */
/*
 * Tcl_AppInit is called from Tcl_Main after the Tcl interpreter
 * has been created, and before the script file or interactive
 * command loop is entered.
 */

int Tcl_AppInit ( Tcl_Interp *interp )
{
  /* Initialize packages */

  if (Tcl_Init(interp) == TCL_ERROR) {
    return(TCL_ERROR);
  }

  if (Tk_Init(interp) == TCL_ERROR) {
    return(TCL_ERROR);
  }

  /*  tcl_version > 7.4
  Tcl_StaticPackage(interp, "Tk", Tk_Init, (Tcl_PackageInitProc *) NULL);
  */

  if (Analog_Init(interp) == TCL_ERROR) {
    return(TCL_ERROR);
  }

  Tcl_SetVar(interp, "tcl_rcFileName", "~/.wishrc", TCL_GLOBAL_ONLY);

  return(TCL_OK);
}

/* ******************************************************* */

static int Analog_Init ( Tcl_Interp *interp )
{
  /* Initialize packages */

  Tcl_CreateCommand(interp, "analog", Tk_AnalogWidgetCreate,
#if ( (TK_MAJOR_VERSION  == 3) )
		    (ClientData) NULL,
#else
		    (ClientData) Tk_MainWindow(interp),
#endif
		    (Tcl_CmdDeleteProc *) NULL);
  
  return(TCL_OK);
}

/* ******************************************************* */
/*  File: tclAnalogAppInit.c                               */
/* ******************************************************* */

